@@ -5,7 +5,7 @@ use Carp;
 use strict;
 use bytes;
 use vars qw($VERSION);
-$VERSION = '2.30';
+$VERSION = '2.31';
 
 use constant RANDOM_DEVICE => '/dev/urandom';
 
@@ -117,7 +117,8 @@ sub new {
 	  unless ($rbs == $bs);
       }
     } else {
-      $padding = $padding eq 'null'           ? \&_null_padding
+      $padding = $padding eq 'none'           ? \&_no_padding
+	        :$padding eq 'null'           ? \&_null_padding
 	        :$padding eq 'space'          ? \&_space_padding
 		:$padding eq 'oneandzeroes'   ? \&_oneandzeroes_padding
 		:$padding eq 'rijndael_compat'? \&_rijndael_compat
@@ -228,6 +229,10 @@ sub crypt (\$$){
 
     my $bs = $self->{'blocksize'};
 
+    croak "When using no padding, plaintext size must be a multiple of $bs"
+      if $self->{'padding'} eq \&_no_padding
+	and length($data) % $bs;
+
     croak "When using rijndael_compat padding, plaintext size must be a multiple of $bs"
       if $self->{'padding'} eq \&_rijndael_compat
 	and length($data) % $bs;
@@ -455,8 +460,8 @@ sub _get_random_bytes {
     $result = pack("C*",map {rand(256)} 1..$length);
   }
   # Clear taint and check length
-  $result =~ /^(.{$length})$/s or croak "Invalid length while gathering $length randim bytes";
-  return $1;
+  length($result) == $length or croak "Invalid length while gathering $length random bytes";
+  return $result;
 }
 
 sub _standard_padding ($$$) {
@@ -481,18 +486,23 @@ sub _space_padding ($$$) {
   return unless length $b;
   $b = length $b ? $b : '';
   if ($decrypt eq 'd') {
-     $b=~ s/ *$//s;
+     $b=~ s/ *\z//s;
      return $b;
   }
   return $b . pack("C*", (32) x ($bs - length($b) % $bs));
 }
 
+sub _no_padding ($$$) {
+  my ($b,$bs,$decrypt) = @_;
+  return $b;
+}
+
 sub _null_padding ($$$) {
   my ($b,$bs,$decrypt) = @_;
   return unless length $b;
   $b = length $b ? $b : '';
   if ($decrypt eq 'd') {
-     $b=~ s/\0*$//s;
+     $b=~ s/\0*\z//s;
      return $b;
   }
   return $b . pack("C*", (0) x ($bs - length($b) % $bs));
@@ -502,9 +512,8 @@ sub _oneandzeroes_padding ($$$) {
   my ($b,$bs,$decrypt) = @_;
   $b = length $b ? $b : '';
   if ($decrypt eq 'd') {
-     my $hex = unpack("H*", $b);
-     $hex =~ s/80*$//s;
-     return pack("H*", $hex);
+     $b=~ s/\x80\0*\z//s;
+     return $b;
   }
   return $b . pack("C*", 128, (0) x ($bs - length($b) % $bs - 1) );
 }
@@ -513,9 +522,8 @@ sub _rijndael_compat ($$$) {
   my ($b,$bs,$decrypt) = @_;
   return unless length $b;
   if ($decrypt eq 'd') {
-     my $hex = unpack("H*", $b);
-     $hex =~ s/80*$//s;
-     return pack("H*", $hex);
+     $b=~ s/\x80\0*\z//s;
+     return $b;
   }
   return $b . pack("C*", 128, (0) x ($bs - length($b) % $bs - 1) );
 }
@@ -672,7 +680,7 @@ The new() method creates a new Crypt::CBC object. It accepts a list of
 
   -padding        The padding method, one of "standard" (default),
                      "space", "oneandzeroes", "rijndael_compat",
-                     or "null" (default "standard").
+                     "null", or "none" (default "standard").
 
   -literal_key    If true, the key provided by "key" is used directly
                       for encryption/decryption.  Otherwise the actual
@@ -968,7 +976,7 @@ Use the 'padding' option to change the padding method.
 
 When the last block of plaintext is shorter than the block size,
 it must be padded. Padding methods include: "standard" (i.e., PKCS#5),
-"oneandzeroes", "space", "rijndael_compat" and "null".
+"oneandzeroes", "space", "rijndael_compat", "null", and "none".
 
    standard: (default) Binary safe
       pads with the number of bytes that should be truncated. So, if 
@@ -996,6 +1004,10 @@ it must be padded. Padding methods include: "standard" (i.e., PKCS#5),
    space: text only
       same as "null", but with "20".
 
+   none:
+      no padding added. Useful for special-purpose applications where
+      you wish to add custom padding to the message.
+
 Both the standard and oneandzeroes paddings are binary safe.  The
 space and null paddings are recommended only for text data.  Which
 type of padding you use depends on whether you wish to communicate
